package com.supervision.manage.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.conditions.update.LambdaUpdateChainWrapper;
import com.supervision.manage.service.DiseaseAncillaryManageService;
import com.supervision.model.DiseaseAncillary;
import com.supervision.service.DiseaseAncillaryService;
import com.supervision.vo.manage.DiseaseAncillaryResVo;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

@Service
@RequiredArgsConstructor
public class DiseaseAncillaryManageServiceImpl implements DiseaseAncillaryManageService {

    private final DiseaseAncillaryService diseaseAncillaryService;

    @Override
    public List<DiseaseAncillaryResVo> queryListByDiseaseId(String diseaseId) {

        Assert.isTrue(StrUtil.isNotEmpty(diseaseId),"diseaseId不能为空");
        List<DiseaseAncillaryResVo> diseaseAncillaryResVos = diseaseAncillaryService.queryListByDiseaseId(diseaseId);

        diseaseAncillaryResVos.forEach(DiseaseAncillaryResVo::populateOther);
        return diseaseAncillaryResVos;
    }

    @Override
    public DiseaseAncillary saveAncillary(DiseaseAncillary diseaseAncillary) {

        assertDiseaseAncillary(diseaseAncillary);

        Assert.isFalse(checkIfPresent(diseaseAncillary),"数据已存在！");

        diseaseAncillaryService.save(diseaseAncillary);

        return diseaseAncillary;
    }

    @Override
    public boolean deleteDiseaseAncillary(String id) {

        Assert.isTrue(StrUtil.isNotEmpty(id),"id 不允许为空");

        return diseaseAncillaryService.removeById(id);
    }

    @Override
    public boolean updateAncillary(DiseaseAncillary diseaseAncillary) {

        Assert.isTrue(StrUtil.isNotEmpty(diseaseAncillary.getId()),"id 不允许为空");

        return diseaseAncillaryService.updateById(diseaseAncillary);
    }

    @Override
    public boolean deleteByDiseaseId(String diseaseId) {
        Assert.notEmpty(diseaseId,"疾病id不能为空");

        return diseaseAncillaryService.lambdaUpdate().eq(DiseaseAncillary::getDiseaseId, diseaseId).remove();
    }


    private void assertDiseaseAncillary(DiseaseAncillary diseaseAncillary){

        Assert.isNull(diseaseAncillary.getId(),"id不为空");

        Assert.isTrue(StrUtil.isNotEmpty(diseaseAncillary.getDiseaseId()),"疾病id不能为空");

        Assert.notNull(diseaseAncillary.getPrimarilyDiagnosisCriteriaFlag(),"初步诊断依据不能为空");

        Assert.notNull(diseaseAncillary.getBasisConfirmFlag(),"证实诊断依据不能为空");

        Assert.notNull(diseaseAncillary.getBasisIdentificationFlag(),"鉴别诊断依据不能为空");

        Assert.notNull(diseaseAncillary.getFullCheckFlag(),"全面诊断不能为空");

        Assert.notNull(diseaseAncillary.getDiagnosisAssessmentFlag(),"判断不能为空");

        Assert.notNull(diseaseAncillary.getRequireCheckFlag(),"是否必查不能为空");

        Assert.notNull(diseaseAncillary.getDiagnosisAssessmentFlag(),"判断不能为空");

        Assert.notNull(diseaseAncillary.getExpectedDiagnosisResult(),"预期诊断结果不能为空");

    }


    private boolean checkIfPresent(DiseaseAncillary diseaseAncillary){

        return !Objects.isNull(findByDiseaseIdAndItemId(diseaseAncillary.getDiseaseId(), diseaseAncillary.getItemId()));
    }


    private DiseaseAncillary findByDiseaseIdAndItemId(String diseaseId,String itemId){
        List<DiseaseAncillary> list = diseaseAncillaryService.lambdaQuery().eq(DiseaseAncillary::getDiseaseId, diseaseId).eq(DiseaseAncillary::getItemId, itemId).list();
        if (CollUtil.isEmpty(list)){
            return null;
        }
        return list.get(0);
    }
}
